#!/usr/bin/env python

# This script will wait for a button to be pressed and then shutdown or reboot the Raspberry Pi.
# A long press initiates a reboot, a very long press initiates a shutdown.
 
import time
from time import sleep
import RPi.GPIO as GPIO
import os
 
# Configure the GPIO pins

def int_active_setup():
Global int_active

demo_mode = False
debug = False
 
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)


# setup the pin to check the shutdown switch - use the internal pull down resistor
GPIO.setup(GPIN, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)

# setup the low battery check pin
GPIO.setup(GPIN2, GPIO.IN)

 
# create a trigger for the shutdown switch and low battery pins
GPIO.add_event_detect(GPIN, GPIO.RISING, callback = shutdown_reboot_callback, bouncetime = 500)

GPIO.add_event_detect(GPIN2, GPIO.FALLING, callback=low_battery_shutdown, bouncetime=500)


# open log file in append mode

Battery_log['logfile_pointer'] = open(Battery_log['logfile'], 'a+')

print "Shutdown / Reboot script started."
 
# ISR: if our button is pressed, we will have a rising edge on pin 21
# this will trigger the interrupt:
def shutdown_reboot_callback(channel):
 
# button is pressed
# possibly shutdown our Raspberry Pi
 
global int_active

 
# only react when there is no other shutdown process running
 
if (int_active == 1):
int_active = 0
pressed = 0
shutdown = 1
 
# count how long the button is pressed
counter = 0
 
while ( pressed == 0 ):
if ( GPIO.input(GPIN) == False ):
# button is still pressed
counter = counter + 1
if debug:
print "pressed: " + str(counter)
# break if we count beyond 20 (long-press is a shutdown)
if (counter >= 20):
pressed = 1
else:
sleep(0.2)
else:
# button has been released
pressed = 1
 
# button has been released, count cycles and determine action
 
# count how long the button was pressed
if (counter < 2):
# short press, do nothing
pressed = 1
int_active = 0
if debug:
print str("short press, do nothing")
 
else:
if debug:
print "else " + str(counter)
# longer press
if (counter < 10):
# medium length press, initiate system reboot
if debug:
print("rebooting..")
 
# run the reboot command as a background process
if demo_mode:
print("sudo reboot now")
exit(0)
else:
os.system("sudo reboot now")
 
elif (counter>=10 and counter<20):
# long press, initiate system shutdown
 
if debug:
print("shutting down..")
 
# run the shutdown command as a background process
if demo_mode:
print("sudo shutdown now")
exit(0)
else:
os.system("sudo shutdown now")


def low_battery_shutdown(channel):
 
# Battery low led flashed
# possibly shutdown our Raspberry Pi
 
global int_active
 
# only react when there is no other shutdown process running
 
if (int_active == 0):
int_active = 1
pressed = 1
shutdown = 0
 
# count how long the led is lit
counter = 0
 
while ( pressed == 1 ):
if ( GPIO.input(GPIN2) == False ):
# led is stil lit
counter = counter + 1
if debug:
print "Low battery warning: " + str(counter)
# break if we count beyond 20 (long-press is a shutdown)
if (counter >= 20):
pressed = 0
else:
sleep(0.2)
else:
# led no longer lit
pressed = 0
 
# button has been released, count cycles and determine action
 
# count how long the led is lit
if (counter < 2):
# short flash, do nothing
pressed = 0
int_active = 0
if debug:
print str("System under load, Voltage is dropping")
 
else:
if debug:
print "else " + str(counter)
# longer press
if (counter < 10):
# medium length flash, initiate warning
if debug:
print("Battery voltage is low, Shutting down soon unless an external powersource is used")
 
# run the reboot command as a background process
if demo_mode:
print()
exit(0)
else:
os.system()
 
elif (counter>=10 and counter<20):
# long flash, initiate system shutdown
 
if debug:
print("Battery voltage too low, Shutting down..")
 
# run the shutdown command as a background process
if demo_mode:
print("sudo shutdown now")
exit(0)
else:
os.system("sudo shutdown now")

 
# Setup int_active variable array


int_active_setup = {}

GPIN=21
GPIN2=15

int_active = 0

Battery_log['logfile'] = '/home/pi/Battery_log.log'  # FULL path to the log file

int_active_setup()


# Although the shutdown is triggered by an interrupt, we still need a loop
# to keep the script from exiting - just do a very long sleep

while True:
if debug:
print "."
time.sleep(1)

int_active_cleanup()